import 'package:germanet_dart/src/synset.dart';

enum CompoundCategory {
  Adjektiv,
  Nomen,
  Verb,
  Adverb,
  Praeposition,
  Partikel,
  Pronomen
}

enum CompoundProperty {
  Abkuerzung,
  Affixiod,
  Fremdwort,
  Konfix,
  Wortgruppe,
  Eigenname,
  opaquesMorphem,
  virtuelleBildung,
  gebundenesMorphem,
  freiesMorphem
}

class CompoundInfo {
  static final PROPERTY = 'property';
  static final CATEGORY = 'category';

  String modifier1;
  String modifier1property;
  WordCategory modifier1category;
  String modifier2;
  String modifier2property;
  WordCategory modifier2category;
  String head;
  String headproperty;

  CompoundInfo(this.modifier1, this.head,
      {this.modifier1property = "",
      this.modifier1category = WordCategory.none,
      this.modifier2 = "",
      this.modifier2property = "",
      this.modifier2category = WordCategory.none,
      this.headproperty = ""}) {
    /*
    This class stores information about a special linguistic entity in German - a compound. A compound
        consists
        of two constituents, a modifier and a head. The head will always be a noun, the modifier can stem from
        a noun,
        a verb or an adjective. Sometimes the modifier can be derived from different words, for example:
        Laufband - modifier: laufen or Lauf. This class stores both possibilities as modifier1 and modifier2.

        :type headproperty: String
        :type modifier2category: WordCategory
        :type modifier2property: String
        :type modifier2: String
        :type modifier1category: WordCategory
        :type modifier1property: String
        :type head: String
        :type modifier1: String
        :param modifier1: A String that represents the modifier (or one possibility). Example: 'Laufband' -
        modifier1:
        'laufen'
        :param head: A String that represents the head. Example: 'Laufband' - head: 'Band'
        :param modifier1property: A String that represents the property of the first modifier variant,
        e.g. if it is
        an Affixoid
        :param modifier1category: The category of the modifier, example: 'laufen' = Verb
        :param modifier2:  A String that represents the modifier (or second possibility). Example: 'Laufband' -
        modifier2:
        'Lauf'
        :param modifier2property: A String that represents the property of the second modifier variant
        :param modifier2category: The category of the second modifier variant, example: 'Lauf' = Nomen
        :param headproperty: A String that represents the property of the head
     */
  }

  @override
  String toString() {
    return "CompoundInfo( modifier = " + modifier1 + ", head = " + head + ")";
  }
}
