import 'package:germanet_dart/src/functools.dart';

class Frames {
    static final String EXPLETIVE = 'NE';
    static final String SUBJECT = 'NN';
    static final String ACCOBJ = 'AN';
    static final String DATOBJ = 'DN';
    static final String GENOBJ = 'GN';
    static final String PREPOBJ = 'PP';
    static final String LOC = 'BL';
    static final String DIR = 'BD';
    static final String TEMP = 'BT';
    static final String MAN = 'BM';
    static final String INST = 'BS';
    static final String CAUSE = 'BC';
    static final String ROLE = 'BR';
    static final String COM = 'BO';
    static final List reflexives = ['DR', 'AR'];

    Map frames2lexunits;

    Frames(this.frames2lexunits);

    Set extract_expletives() => extract_specific_complements(EXPLETIVE);
    
    Set extract_accusative_complement() => extract_specific_complements(ACCOBJ);

    Set extract_dative_complement() => extract_specific_complements(DATOBJ);

    Set extract_gentive_complement() => extract_specific_complements(GENOBJ);

    Set extract_prepositional_complement() => extract_specific_complements(PREPOBJ);

    Set extract_reflexives() => extract_specific_complements(reflexives[0]).union(extract_specific_complements(reflexives[1]));

    Set extract_adverbials() => extract_specific_complements(LOC)
            .union(extract_specific_complements(DIR)
                   .union(extract_specific_complements(TEMP)
                          .union(extract_specific_complements(MAN)
                                 .union(extract_specific_complements(INST)
                                        .union(extract_specific_complements(CAUSE)
                                               .union(extract_specific_complements(ROLE)
                                                      .union(extract_specific_complements(COM))))))));
    
    Set extract_transitives() => extract_specific_complements(ACCOBJ)
            .union(extract_specific_complements(DATOBJ)
                   .union(extract_specific_complements(GENOBJ)
                          .union(extract_specific_complements(PREPOBJ))));

    

    Set extract_intransitives(){
      Set transitives = extract_transitives();
      
      //impl new instead of reduce
      Set add_verbs = Set();
      Iterable it = frames2lexunits.values;
      for(Set s in it){
        add_verbs.union(s);
      }
      return add_verbs.difference(transitives);
    }

    Set extract_specific_complements(String complement){

      Set complements = Set();
      for (var key in frames2lexunits.keys.toList()) {
        if(key.contains(complement)){
          for (var unit in frames2lexunits[key]) {
            complements.add(unit);
          }
        }
      }
      return complements;

    }


}