import 'dart:collection';

import 'package:germanet_dart/germanet_dart.dart';
import 'package:germanet_dart/src/functools.dart';
import 'package:germanet_dart/src/synset.dart';

List _get_overall_longest_shortest_distance(Germanet germanet, WordCategory category){
  List<Synset> synsets = germanet.getSynsetsByWordcategory(category);
  List longest_shortest_distances = [];
  for (Synset synset in synsets) {
    var distances = synset.get_distances_hypernym_dic();
    var longest_shortest_distance = distances.values.toList().fold(distances.values.first, (previousValue, element) => (previousValue as int < (element as int) ? (element) : previousValue));
    longest_shortest_distances.add(longest_shortest_distance);
  }
  var overall_maxlen = longest_shortest_distances.fold(longest_shortest_distances.first, (previousValue, element) => (previousValue as int < (element as int) ? (element) : previousValue));
  Map<Synset, int> dist_dic = directZipMap(synsets, longest_shortest_distances).cast();
  longest_shortest_distances = dist_dic.values.toList();
  longest_shortest_distances.sort();
  longest_shortest_distances = longest_shortest_distances.reversed.toList();
  /**
   * longest_shortest_distances is sorted numberly (9,8,7,..)
   * 
   * the longest_shortest_distances list equals the values of dist_dic while the keys are not connected to the values in this case
   * because the sorting was just applied on the values
   * 
   * solution: sorting the complete map.
   */
  //var sortByValue = SplayTreeMap<Synset, int>.of(dist_dic, ((key1, key2) => dist_dic[key1]!.compareTo(dist_dic[key2]!)));

  var sortedKeys = dist_dic.keys.toList(growable:false)
    ..sort((k1, k2) => dist_dic[k1]!.compareTo(dist_dic[k2]!));
    LinkedHashMap sortedMap = LinkedHashMap
      .fromIterable(sortedKeys.reversed, key: (k) => k, value: (k) => dist_dic[k]);
    

  return [sortedMap, overall_maxlen];
}

int _get_greatest_depth(Germanet germanet, WordCategory category){

  List<Synset> synsets = germanet.getSynsetsByWordcategory(category);
  int max_depth = 0;
  print(synsets.length);
  for (Synset synset in synsets) {
    int depth = synset.min_depth();
    if(depth >= max_depth){
      max_depth = depth;
    }
  }
  return max_depth;

}

List get_longest_possible_shortest_distance(Germanet germanet, WordCategory category){

  List l0 = _get_overall_longest_shortest_distance(germanet, category);
  Map sorted_dist_dic = l0[0];
  int overall_maxlen = l0[1];

  int longest_possible_shortest_distance = 0;

  List synset_pair_longest_distance = [germanet.root(), germanet.root()];

  print(sorted_dist_dic.length);

  for (Synset synset in sorted_dist_dic.keys) {
    var longest_shortest_dist = sorted_dist_dic[synset];
    if(longest_shortest_dist + overall_maxlen <= longest_possible_shortest_distance){
      continue;
    }
    for (Synset current in sorted_dist_dic.keys) {
      var current_shortest_dist = sorted_dist_dic[current];
      if(current_shortest_dist + longest_shortest_dist <= longest_possible_shortest_distance){
        continue;
      }
      var pathdist = current.shortest_path_distance(synset);
      if(pathdist > longest_possible_shortest_distance){
        longest_possible_shortest_distance = pathdist;
        synset_pair_longest_distance = [synset, current];
      }
    }
  }
  return [longest_possible_shortest_distance, overall_maxlen, synset_pair_longest_distance];
}

