import 'package:germanet_dart/src/lexunit.dart';

enum ConRel {

    has_hypernym(1),    has_hyponym(2),    has_component_meronym(3),    has_component_holonym(4),    has_member_meronym(5),    has_member_holonym(6),    has_substance_meronym(7),    has_substance_holonym(8),    has_portion_meronym(9),    has_portion_holonym(10),    entails(11),    is_entailed_by(12),    is_related_to(13),    causes(14);
  const ConRel(this.value);
  final int value;

  bool transitive(ConRel conRel){

    List transitive_rels = [1,2,3,4,5,6,7,8,9,10];
    return transitive_rels.contains(conRel.value);

  }
}

enum WordCategory{
  none(0),
  adj(1),
  nomen(2),
  verben(3);
  
  const WordCategory(this.value);
  final int value;

  static Set get_possible_word_classes(WordCategory wordCategory){
    if(wordCategory.value == 2){
      return {
        WordClass.Tops, WordClass.Artefakt, WordClass.Attribut, WordClass.Besitz, WordClass.Relation,
                    WordClass.Geschehen, WordClass.Form, WordClass.Gefuehl, WordClass.Gruppe, WordClass.Koerper,
                    WordClass.Kognition, WordClass.Kommunikation, WordClass.Menge, WordClass.Mensch, WordClass.Motiv,
                    WordClass.Nahrung, WordClass.natGegenstand, WordClass.Ort, WordClass.Pflanze, WordClass.Substanz,
                    WordClass.Tier, WordClass.Zeit
      };
    }
    if(wordCategory.value == 1){
      return {WordClass.Allgemein, WordClass.Bewegung, WordClass.Relation, WordClass.Gefuehl,
                    WordClass.Gesellschaft, WordClass.Koerper, WordClass.Geist, WordClass.Menge,
                    WordClass.natGegenstand, WordClass.Ort, WordClass.Pertonym, WordClass.Substanz, WordClass.Verhalten,
                    WordClass.Perzeption, WordClass.Zeit, WordClass.privativ};
    }
    else{
      return {WordClass.Allgemein, WordClass.Besitz, WordClass.Gefuehl, WordClass.Gesellschaft,
                    WordClass.Koerperfunktion, WordClass.Kognition, WordClass.Kommunikation, WordClass.Konkurrenz,
                    WordClass.Kontakt, WordClass.natPhaenomen, WordClass.Lokation, WordClass.Schoepfung,
                    WordClass.Veraenderung, WordClass.Verbrauch, WordClass.Perzeption};
    }
  }
}

enum WordClass {
    None(0),
    Allgemein(1),    Bewegung(2),    Gefuehl(3),    Geist(4),    Gesellschaft(5),    Koerper(6),    Menge(7),    natPhaenomen(8),    Ort(9),    Pertonym(10),    Perzeption(11),    privativ(12),    Relation(13),    Substanz(14),    Verhalten(15),    Zeit(16),    Artefakt(17),    Attribut(18),    Besitz(19),    Form(20),    Geschehen(21),    Gruppe(22),    Kognition(23),    Kommunikation(24),    Mensch(25),    Motiv(26),    Nahrung(27),    natGegenstand(28),    Pflanze(29),    Tier(30),    Tops(31),    Koerperfunktion(32),    Konkurrenz(33),    Kontakt(34),    Lokation(35),    Schoepfung(36),    Veraenderung(37),    Verbrauch(38);

    const WordClass(this.value);
    final int value;

    static Set get_possible_word_categories(WordClass wordClass){

      Set categories = {};
      Set verb_classes = {37, 5, 1, 32, 3, 35, 11, 33, 38, 36, 19, 34, 8, 24, 23};
      Set noun_classes = {26, 31, 14, 29, 7, 22, 24, 6, 9, 16, 23, 28, 3, 17, 20, 30, 27, 25, 19, 13, 18, 21};
      Set adj_classes = {23, 32, 24, 19, 8, 11, 35, 33, 3, 34, 1, 37, 38, 5, 36};
      if(verb_classes.contains(wordClass.value)){
        categories.add(WordCategory.verben);
      }
      if(noun_classes.contains(wordClass.value)){
        categories.add(WordCategory.nomen);
      }
      if(adj_classes.contains(wordClass.value)){
        categories.add(WordCategory.adj);
      }
      return categories;
    }
}

class Synset {

  var id;
  WordCategory category;
  WordClass word_class;
  late String paraphrase;
  late List<Lexunit> lexunits;
  late Map relations;
  late Map incoming_relations;

  dynamic direct_hypernyms(){
    _direct_hypernyms ??= relations[ConRel.has_hypernym];
    return _direct_hypernyms;
  }
  dynamic direct_hyponyms(){
    _direct_hyponyms ??= relations[ConRel.has_hyponym];
    return _direct_hyponyms;
  }
  late dynamic _direct_hypernyms;
  late dynamic _direct_hyponyms;

  Synset(this.id, this.category, this.word_class){

    paraphrase = "";
    lexunits = [];
    relations = {};
    incoming_relations = {};

    _direct_hypernyms = null;
    _direct_hyponyms = null;

  }

  @override
  String toString() {
    List<String> lex_list = [];
    for (Lexunit lex in lexunits) {
      lex_list.add(lex.orthform);
    }
    return "Synset(id="+id+"), lexunits={"+lex_list.join(",")+"})";
  }

  void add_lexunit(Lexunit unit){
    lexunits.add(unit);
  }

  bool is_root() => direct_hypernyms() == null || (direct_hypernyms() as Iterable).isEmpty;

  bool is_leaf() => direct_hyponyms() == null || (direct_hyponyms() as Iterable).isEmpty;

  int num_lexunits() => lexunits.length;

  List _hypernym_paths(){

    _direct_hypernyms = relations[ConRel.has_hypernym];
    _direct_hyponyms = relations[ConRel.has_hyponym];

    List paths = [];
    dynamic hypernyms = direct_hypernyms();
    if(is_root()){
      return [[this]];
    }
    for(dynamic hypernym in hypernyms){

      for(List ancestor_list in hypernym._hypernym_paths()){

        ancestor_list.add(this);
        paths.add(ancestor_list);

      }

    }
    return paths;

  }

  Set _all_hypernyms(){

    List hypernyms = [];
    for(dynamic path in _hypernym_paths()){

      for(Synset synset in path){

        if(synset != this){

          hypernyms.add(synset);

        }

      }

    }
    return hypernyms.toSet();

  }

  List _hyponym_paths(){

    List paths = [];
    dynamic hypernyms = this.direct_hyponyms;
    if(is_root()){
      paths = [this];
    }
    for(dynamic hypernym in hypernyms){

      for(List ancestor_list in hypernym._hyponym_paths()){

        ancestor_list.add(this);
        paths.add(ancestor_list);

      }

    }
    return paths;

  }

  Set _all_hyponyms(){

    List hypernyms = [];
    for(dynamic path in _hyponym_paths()){

      for(Synset synset in path){

        if(synset != this){

          hypernyms.add(synset);

        }

      }

    }
    return hypernyms.toSet();

  }

  List _shortest_path_to_root(){

    List paths = _hypernym_paths();
    int m = paths.fold(paths.first.length, (previousValue, element) => element.value < previousValue ? element.value : previousValue);
    return paths.firstWhere((element) => element.length == m);

  }

  Set _common_hypernyms(Synset other){

    return _all_hypernyms().toSet().intersection(other._all_hypernyms().toSet());
  }

  /// min_depth (to root) -> minimal distance to root from this synset
  int min_depth(){
    
    dynamic hypernyms = relations[ConRel.has_hypernym];
    int min_depth = 0;
    if(hypernyms == null){
      min_depth = 0;
    }else{
      //slow activity... -> optimizing [fixed]
      List x = (hypernyms as List).map((e) => e.min_depth()).toList();
      x.sort();
      min_depth = x[0] + 1;
    }
    return min_depth;

  }

  /// shortest distance (int) to another Synset in the word tree
  int shortest_path_distance(Synset other){

    if(other == this){
      return 0;
    }

    List paths = _shortest_path(other);
    if(paths == []){
      return 0;
    }
    return paths[0].length - 1;

  }

  List _shortest_path(Synset other){

    List shortest_paths = [];
    Set lcs = lowest_common_subsumer(other);
    for(dynamic subsumer in lcs){
      
      List start_to_lcs_paths = _shortest_path_to_hypernym(subsumer);
      List end_to_lcs_paths = other._shortest_path_to_hypernym(subsumer);
      for(var start_path in start_to_lcs_paths){
                for(var end_path in end_to_lcs_paths){
                    List current_path = List.from(start_path);
                    List l = end_path.toList();
                    end_path = l.reversed;

                    for(Synset synset in (end_path)){
                        if(synset != subsumer){
                            current_path.add(synset);
                        }
                    }
                    if(!shortest_paths.contains(current_path)){
                        shortest_paths.add(current_path);
                    }
                }
      }
    }

    return shortest_paths;

  }

  List _shortest_path_to_hypernym(Synset hypernym){

    if(this == hypernym){
      return [[this]];
    } 

    List shortest_path = [];
    int shortest = 0x7fffffffffffffff;

    for (List path in _hypernym_paths()) {
      if(path.contains(hypernym)){
        int index = path.indexOf(hypernym);
        var current_path = path.getRange(index, path.length);
        int path_len = current_path.length;
        if(path_len <= shortest){

          shortest = path_len;
          current_path = current_path.toList().reversed;
          shortest_path.add(current_path);

        }
      }
    }
      List l0 = shortest_path.map((e) => e.length).toList();
      l0.sort();
      int shortest_dist = l0[0];
      return shortest_path.where((element) => element.length == shortest_dist).toList();
  }

  
  /// returning the first point where the hypernym path of both synsets match
  Set lowest_common_subsumer(Synset other){

    Set lcs = {};
    if(other == this){
      lcs.add(this);
      return lcs;
    }
    if(direct_hypernyms().contains(other) || other.is_root()){
      lcs.add(other);
      return lcs;
    }
    if(direct_hypernyms().contains(this) || is_root()){
      lcs.add(this);
      return lcs;
    }
    Set common_hypernyms = this._common_hypernyms(other);
    Map dist_dict1 = get_distances_hypernym_dic();
    Map dist_dict2 = other.get_distances_hypernym_dic();
    int dist = 0x7fffffffffffffff;
    for (var hypernym in common_hypernyms) {
        var dist1 = dist_dict1[hypernym];
        var dist2 = dist_dict2[hypernym];
        if(dist1 + dist2 < dist){
          lcs.clear();
          lcs.add(hypernym);
          dist = dist1 + dist2;
        }
        if(dist1 + dist2 == dist){
          lcs.add(hypernym);
        }
    }
    return lcs;
  }

  /// internal method, not needed, do not call this directly
  Map get_distances_hypernym_dic(){
    List hypernym_paths = this._hypernym_paths();
    Map distances_dic = {};
    for (var p in hypernym_paths) {
      for (var i = 0; i < p.length; i++) {
          var hypernym = p[i];
          int dist = p.length - 1 - i;
          if(distances_dic.keys.contains(hypernym)){
              int current_dist = distances_dic[hypernym];
              if(dist < current_dist){
                distances_dic[hypernym] = dist;
              }
          }else{
            distances_dic[hypernym] = dist;
          }
      }
    }
    return distances_dic;
  }



  







}